function r = cophenet_bootstrap(A,d,nloop,verbose)
%Haixuan Yang modified the function nmfconsensus on Aug. 15, 2014.
%There are two outputs: consensus is  3d array of consensus matrices by the
%basic NMF; consensus1 is the consensus matrices by proposed one.
%
%Calculate the cophenet according to Algorithm 2 in the paper
%Haixuan Yang, and Cathal Seoighe. A simple post-processing method of nonnegative matrix factorization for molecular cancer class discovery.
%
%A: an non-negative gene expression matrix , where rows represent genes and columns samples;
%d: the number of clusters;
%nloop : number of initial conditions
%
%verbose : prints iteration count if not set to 0
%

% test for negative values in v
StartUp;
if(min(A(:)) < 0)
    disp('matrix entries can not be negative');
    return;
end
if(min(sum(A,2)) == 0)
    disp('not all entries in a row can be zero');
    return;
end

[n,m]=size(A);
conn1=zeros(m,m); %HX. Aug 15, 2014

for(loop=1:nloop)
    if (verbose)
        fprintf(' iter %d\n',loop);
    end
    ind = randsample(n,n,true); %using gene sets sampled uniformly at random with replacement from the whole gene sets
    h1=nomarlization_auto6_rand(A(ind,:),d,'KL',2); %HX. Oct. 2, 2014
    conn1=conn1+nmfconnectivity(h1);%HX. Aug. 15, 2014; %compute consensus matrix
end

conn1=conn1/nloop; %HX. Aug. 15, 2014

Y=upper_diag2row(conn1);
Z = linkage(Y,'average');
r = cophenet(Z,Y);
end

function conn= nmfconnectivity(h)
%
% Jean-Philippe Brunet
% Cancer Genomics 6/10/03
%
mm=size(h);
k=mm(1);
m=mm(2);

% compute m x m matrix which is 1 if samples are together, 0 elsewhere

% determine sample assignment by its largest metagene expresion value
[y,index]=max(h,[],1);

mat1=repmat(index,m,1); % spread index down
mat2=repmat(index',1,m); % spread index right

conn=mat1==mat2; % 1 when for pair of samples with same assignement
end

function h=nomarlization_auto6_rand(A,d,type,num)
%HX. Dec 16, 2014
if(nargin<4)
    num=2;
end
[w,h]=best_nmf(A,d,num,type);
%%
di=max(w,[],1);
h=diag(di)*h;
end



