function [w,h]=pNMF(A,d,type,normalization)
%Haixuan Yang. Dec 15, 2014.
%pNMF is a post-processing method of a basic NMF. The basic idea is to normalize w so that columns of w have equal norms, 
%here A ~ wh is found by a basic NMF. For more details, see the paper: 
%Haixuan Yang, and Cathal Seoighe. A simple post-processing method of nonnegative matrix factorization for molecular cancer class discovery.
%Usage
 %[w,h]=pNMF(A,d,type,normalization)
%A: an non-negative gene expression matrix , where rows represent genes and columns samples;
%d: the number of clusters;
%type: either 'KL' or 'EU', indicateing the Basic NMF is the one minimizing a KL divergence or Euclidean distance.
%normalization: if it is smaller than 1, then it means quantile; otherwise, it means p-norm.
%The default version reported in the paper 
%[w,h]=pNMF(A,d);
%which means type='KL', normalization = inf -- simply to put the maximum values (the infinity norm) of columns of w equal
%
if(nargin<3)
    type='KL';
end
if(nargin<4)
    normalization=inf;
end
num=2;
%%filter
[w,h]=best_nmf(A,d,num,type);
d=size(w,2);
di=get_norm(w,j);
w1=w*diag(1./di);
rank=max(w1,[],2)-min(w1,[],2);
rem_ind=find(rank>median(rank));
A=A(rem_ind,:);
%%
[w,h]=best_nmf(A,d,num,type);
di=get_norm(w,normalization);
h=diag(di)*h;
end

function Di=get_norm(W,normalization)
m=size(W,2);
Di=zeros(m,1);
if(normalization<1)
    for(i=1:m)
        w=W(:,i);
        if(normalization<1)
            Di(i)=quantile(w,normalization);
        else
            Di(i)=norm(w,normalization);
        end
    end%i
end
end